#ifndef COMMON_H
#define COMMON_H

enum {FALSE=0,TRUE=1};
typedef unsigned char bool; 

#define max(a,b) (((a)>(b))?(a):(b))
#define min(a,b) (((a)<(b))?(a):(b))

#include "error/error.h"
#include "common/setup.h"
#include <stdlib.h>


/************************************
 *  Useful structures and handlers  *
 ************************************/

// Vector of longs
//-----------------

typedef struct LongVec_ 
{
    long size;
    long top;
    long* cont;
} LongVec;

LongVec* longVecNew(long);
int longVecInit(LongVec *,long);
int longVecEnd(LongVec *);
int longVecDestroy(LongVec *);
int longVecCopy(LongVec *,LongVec *);
int longVecPushBack(LongVec *,long);
int longVecPopBack(LongVec *,long *);
int longVecResize(LongVec *,long);
int longVecGet(LongVec *,long,long *);
int longVecSet(LongVec *,long,long);


// List of longs
//---------------

typedef LongVec LongList; 

#define longListNew(a) longVecNew(a)
#define longListInit(a,b) longVecInit((LongVec*) a,b)
#define longListEnd(a) longVecEnd((LongVec*) a)
#define longListDestroy(a) longVecDestroy((LongVec*) a)
#define longListCopy(a,b) longVecCopy((LongVec*) a,(LongVec*) b)
#define longListResize(a,b) longVecResize((LongVec*) a,b)
#define longListPushBack(a,b) longVecPushBack((LongVec*) a,b)
#define longListPopBack(a,b) longVecPopBack((LongVec*) a,b)
int longListInsert(LongList *,long,long);
int longListRemove(LongList *,long);
int longListMerge(LongList *,LongList *,LongList *);
int longListSort(LongList *);

// Vector of doubles
//-------------------

typedef struct DoubleVec_ 
{
    long size;
    long top;
    double* cont;

} DoubleVec;

DoubleVec* dblVecNew(long);
int dblVecInit(DoubleVec *,long);
int dblVecEnd(DoubleVec *);
int dblVecDestroy(DoubleVec *);
int dblVecCopy(DoubleVec *,DoubleVec *);
int dblVecPushBack(DoubleVec *,double);
int dblVecPopBack(DoubleVec *,double *);
int dblVecResize(DoubleVec *,long);
int dblVecGet(DoubleVec *,long,double *);
int dblVecSet(DoubleVec *,long,double);


/******************************
 *   Allocators redifinition  *
 ******************************/

extern long allocatedMemory;

#define malloc(a) mymalloc(a)
#define realloc(a,b) myrealloc(a,b)
#define free(a) myfree(a)
#define calloc(a,b) mycalloc(a,b)

void *mymalloc(size_t);
void *myrealloc(void*,size_t);
void myfree(void*);
void *mycalloc(size_t,size_t);

#define ANDERROR(a,b) (a=((b)<0?-1:(a)))


#endif
