#ifndef DATABASE_API_H
#define DATABASE_API_H

#include "common/common.h"
#include "common/setup.h"
#include "error/error.h"
#include "database/database_st.h"
#include "computation/computation_st.h"
#include "predicate/predicate.h"
#include "rerouting/rerouting.h"

/*******************************************
 * Communication structures accessors      *
 *******************************************/

// Label switched path representation
//------------------------------------

DBLabelSwitchedPath* DBlspNew();
int DBlspInit(DBLabelSwitchedPath*);
int DBlspDestroy(DBLabelSwitchedPath*);
int DBlspEnd(DBLabelSwitchedPath*);
int DBlspCopy(DBLabelSwitchedPath*,DBLabelSwitchedPath*);


// Ordered List of LSPs
//----------------------

DBLSPList* DBlspListNew(long);
int DBlspListInit(DBLSPList*,long);
int DBlspListDestroy(DBLSPList*);
int DBlspListEnd(DBLSPList*);
int DBlspListInsert(DBLSPList*,DBLabelSwitchedPath*);
int DBlspListRemove(DBLSPList*,DBLabelSwitchedPath*);
int DBlspCompare(const DBLabelSwitchedPath*,const DBLabelSwitchedPath*);


// Link state representation
//---------------------------

DBLinkState* DBlinkStateNew();
int DBlinkStateInit(DBLinkState*);
int DBlinkStateDestroy(DBLinkState*);
int DBlinkStateEnd(DBLinkState*);
int DBlinkStateCopy(DBLinkState*,DBLinkState*);

int DBevalLSOnSetup(DataBase*, long, long, DBLinkState*, DBLinkState*, LSPRequest*);
int DBevalLSOnRemove(DataBase*, long, long, DBLinkState*, DBLinkState*, LSPRequest*);

int DBupdateLSOnSetup(DataBase*, long, long, DBLinkState*, DBLabelSwitchedPath*);
int DBupdateLSOnRemove(DataBase*, long, long, DBLinkState*, DBLabelSwitchedPath*);

/**********************
 *    database API:   *
 **********************/

DataBase *DBnew(long);
int DBdestroy(DataBase *);

long DBgetID(DataBase *);
long DBgetLinkID(DataBase *,long,long);
long DBgetLinkSrc(DataBase *,long);
long DBgetLinkDst(DataBase *,long);

long DBgetNbNodes(DataBase *);
long DBgetMaxNodeID(DataBase *);
int DBaddNode(DataBase *,long);
int DBremoveNode(DataBase *,long);

long DBgetNbLinks(DataBase *);
int DBaddLink(DataBase *,long,long,long,DBLinkState *);
int DBremoveLink(DataBase *,long,long);

int getpreemptList(DataBase *,DBLabelSwitchedPath*,LongList *);
int DBaddLSP(DataBase *,DBLabelSwitchedPath*,LongList*);
int DBremoveLSP(DataBase *,long);
DBLabelSwitchedPath* DBgetLSP(DataBase *,long);
DBLSPList* DBgetLinkLSPs(DataBase *,long,long);

DBLinkState* DBgetLinkState(DataBase *,long,long);
int DBsetLinkState(DataBase *,long,long,DBLinkState*);

LongList* DBgetNodeInNeighb(DataBase *,long);
LongList* DBgetNodeOutNeighb(DataBase *,long);

void DBprintDB(DataBase*);


#endif
