#ifndef DATABASE_UTIL_H
#define DATABASE_UTIL_H

#include "database/database_st.h"

/******************************************************************
 *    Structures and substructures of the database and handlers   * 
 ******************************************************************/


//Internal representation of a node in the DB
//--------------------------------------------

typedef struct DBNode_ 
{
    long id;
    /// List of Nodes which have a link toward this Node.
    LongList inNeighb;
    /// List of Nodes towards which this Node has a link.
    LongList outNeighb;
} DBNode;

DBNode* DBnodeNew();
int DBnodeEnd();
int DBnodeInit(DBNode*);
int DBnodeDestroy(DBNode*);
void DBprintNode(DBNode*);



//Internal representation of a link in the DB
//--------------------------------------------

typedef struct DBLink_
{
    long id;
    DBLinkState state;
    /// List of LSPs passing through the Link.
    DBLSPList lspList;
} DBLink;

DBLink* DBlinkNew();
int DBlinkEnd();
int DBlinkInit(DBLink*);
int DBlinkDestroy(DBLink*);
void DBprintLink(DBLink*);



// Collection of all nodes (pointers) representation
//---------------------------------------------------

typedef struct DBNodeVec_
{
    long size;
    long top;
    DBNode **cont;
} DBNodeVec;

DBNodeVec* DBnodeVecNew(long);
int DBnodeVecInit(DBNodeVec*,long);
int DBnodeVecDestroy(DBNodeVec*);
int DBnodeVecEnd(DBNodeVec*);
int DBnodeVecResize(DBNodeVec*,long);
int DBnodeVecSet(DBNodeVec*,DBNode*,long);
int DBnodeVecRemove(DBNodeVec*,long);
#define DBnodeVecGet(a,b) (((b)>=((a)->size))?NULL:(((DBNodeVec*)(a))->cont[b]))


// Collection of all LSPs (pointers) representation
//--------------------------------------------------

typedef struct DBLSPVec_
{
    long size;
    DBLabelSwitchedPath **cont;
} DBLSPVec;

DBLSPVec* DBlspVecNew(long);
int DBlspVecInit(DBLSPVec*,long);
int DBlspVecDestroy(DBLSPVec*);
int DBlspVecEnd(DBLSPVec*);
int DBlspVecResize(DBLSPVec*,long);
int DBlspVecSet(DBLSPVec*,DBLabelSwitchedPath*,long);
int DBlspVecRemove(DBLSPVec*,long);
#define DBlspVecGet(a,b) (((b)>=((a)->size))?NULL:(((DBLSPVec*)(a))->cont[b]))



// Collection of all links (pointers) representation
//----------------------------------------------------

typedef struct DBLinkTab_
{
    long size;
    DBLink ***cont;
} DBLinkTab;

DBLinkTab* DBlinkTabNew(long);
int DBlinkTabInit(DBLinkTab*,long);
int DBlinkTabDestroy(DBLinkTab*);
int DBlinkTabEnd(DBLinkTab*);
int DBlinkTabResize(DBLinkTab*,long);
int DBlinkTabSet(DBLinkTab*,DBLink*,long,long);
int DBlinkTabRemove(DBLinkTab*,long,long);
#define DBlinkTabGet(a,b,c) ((((b)>=(a)->size)||((c)>=(a)->size))?NULL:((a)->cont[b][c]))




// Global structure containing all the information (Topology, Status, LSP, ...)
//------------------------------------------------------------------------------

struct DataBase_
{
    /** 
     * ID of the node to which this database is related. 
     * Used to garantee that when this agent is used in simulator mode, 
     * no illegal information is accessed.
     */
    long id;
    long nbNodes;
    long nbLinks;
    /// Array of all nodes.
    DBNodeVec nodeVec;
    /// Array of all LSPs established.
    DBLSPVec lspVec;
    /// Bidimentionnal array of Links. It is #nodeVec * #nodeVec large.
    DBLinkTab linkTab;
    LongVec linkSrcVec;
    LongVec linkDstVec;
};


#endif
