#include "rerouting.h"
#include <stdio.h>
#include <math.h>


int chooseReroutedLSPs(int precedence,DBLinkState *state,DBLSPList *lspList,double toGain[NB_OA],
		       LongList *idList)
{
    int i,j,p;
    double need;    

    if (idList==NULL || idList->cont==NULL || state==NULL ||
	lspList==NULL || lspList->cont==NULL)
    {
	addError(CRITICAL,"Invalid arguments in %s at line %d",
		     __FILE__,__LINE__);
	return -1;
    }
    
    for (i=1;i<NB_OA;i++)
    {
	if (toGain[i]>DIGIT_PRECISION)
	{
	    addError(CRITICAL,"Only one ordered aggregate taken into account in %s at line %d",
		     __FILE__,__LINE__);
	    return -1;
	}
	for (j=0;j<NB_PREEMPTION;j++)
	{
	    if (state->rbw[i][j]>DIGIT_PRECISION)
	    {
		addError(CRITICAL,"Only one ordered aggregate taken into account in %s at line %d",
			 __FILE__,__LINE__);
		return -1;
	    }
	}
    }
    
    need=toGain[0];
    j=0;
    while (need>DIGIT_PRECISION && j<lspList->top)
    {
	p=lspList->cont[j]->precedence;
	if (p<=precedence)
	{
	    addError(CRITICAL,"Not enough preemptable bandwidth for LSP selection in %s at line %d",
		     __FILE__,__LINE__);
	    return -1; 
	}
	
	if (need >= state->rbw[0][p])
	{
	    while (j<lspList->top && lspList->cont[j]->precedence==p)
	    {
		longListPushBack(idList,lspList->cont[j]->id);
		j++;
	    }
	    need=need-state->rbw[0][p];
	    p=p-1;
	}
	else while (need>DIGIT_PRECISION && j<lspList->top)
	{
	    if (lspList->cont[j]->bw[0]<=need)
	    {
		longListPushBack(idList,lspList->cont[j]->id);
		need=need-lspList->cont[j]->bw[0];
		j++;
	    }
	    else
	    {
		while (j<lspList->top && lspList->cont[j]->bw[0]>=need && lspList->cont[j]->precedence==p)
		    j++;
		longListPushBack(idList,lspList->cont[j-1]->id);
		need=need-lspList->cont[j-1]->bw[0];
	    }
	}
    }

    if (need>DIGIT_PRECISION)
    {
	addError(CRITICAL,"Not enough preemptable bandwidth for LSP selection in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    return 0;
}


double makeRerouteScore(LSPRequest *req,double gain[NB_OA],DBLinkState *ls,int oa)
{
    int curPrec;
    double bwGained,score=0;
    
    bwGained=0;
    curPrec=NB_PREEMPTION-1;
    while (bwGained<gain[oa])
    {
	if (curPrec<=req->precedence)
	{
	    addError(CRITICAL,"internal error: impossible to realize gain in %s at line %d",
		     __FILE__,__LINE__);
	    return HUGE_VAL;
	}
	score=score+damoteConfig.reroutingConfig.scoreCoef[oa][curPrec]*
	    min(ls->rbw[oa][curPrec],gain[oa]-bwGained);	    
	bwGained=bwGained+
	    min(ls->rbw[oa][curPrec],gain[oa]-bwGained);
	curPrec--;
    }
    
    return score/req->bw[oa];
}
