#include "database/database_api.h"
#include "computation/computation_st.h"
#include "computation/backup/backup_api.h"
#include "common/common.h"
#include "error/error.h"

#include <stdio.h>
#include <string.h>

#define LINUX
#define TIME1

#if defined LINUX && defined TIMING
#include <sys/time.h>
#endif

int
main() 
{
    DataBase* centralDB=NULL;
    DBLinkState* ls=NULL;
    long i;
    int src[] =    {0 ,1 ,0 ,1 ,1 ,2 ,2 ,3 ,10,3 ,4 ,4 ,6 ,7 ,7 ,5 ,5 ,8};
    int dst[] =    {1 ,10,2 ,2 ,3 ,3 ,5 ,5 ,4 ,4 ,6 ,7 ,7 ,5 ,8 ,8 ,9 ,9};
    double cap[] = {10,10,10,10,20,20,20,20,10,20,10,10,10,20,10,10,10,10};
    DBLabelSwitchedPath* lsp=NULL;
    LSPRequestList req;
    LongVec rerouting;

#if defined LINUX && defined TIMING && defined TIME1
    struct timezone tz;
    struct timeval  t1,t2;
#endif

    // Init the error manager ....
    errorInit();

    // Allocate a new DB
    if ((centralDB = DBnew(0)) == NULL)
    {
	addError(CRITICAL,"Impossible to init a DataBase in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }
   
    // ------------------- TEST 1 -----------------------
    // Create the topology
    
    printf("Creating Nodes ...\n");
    for (i=0; i<11; i++)
    {
	if (DBaddNode(centralDB, i) != 0)
	{
	    addError(CRITICAL,"Impossible to init a Node in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}
    }
   
    printf("Init of the default linkstate\n");
    if ((ls = DBlinkStateNew()) == NULL)
    {
	addError(CRITICAL,"Impossible to init a Link in %s at line %d",
		 __FILE__,__LINE__);
  	goto clean;
    }

    printf("Adding Links to the topology\n");
    for (i=0; i<18; i++)
    {
	ls->cap[0] = cap[i];
	ls->cap[1] = 0.3*cap[i];
	ls->color = 0;
	
	printf("Adding Link %ld\n", i);
	
	if (DBaddLink(centralDB, 2*i, src[i], dst[i], ls) != 0) 
	{
	    addError(CRITICAL,"Impossible to add Link in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}

	if (DBaddLink(centralDB, 2*i+1, dst[i], src[i], ls) != 0) 
	{
	    addError(CRITICAL,"Impossible to add Link in %s at line %d",
		     __FILE__,__LINE__);
	    goto clean;
	}
  
    }

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);
    
    // Clear the ls
    DBlinkStateDestroy(ls);

    // new LSP
    lsp = DBlspNew();

    // primary
    lsp->id = 0;
    lsp->bw[0] = 1;
    lsp->bw[1] = 2;
    lsp->precedence = 0;
    lsp->type = PRIM;
    lsp->primID = -1;
    lsp->noContentionId = -1;

    longListPushBack(&(lsp->path), 0);
    longListPushBack(&(lsp->path), 2);
    longListPushBack(&(lsp->path), 3);
    longListPushBack(&(lsp->path), 5);
    longListPushBack(&(lsp->path), 7);
    longListPushBack(&(lsp->path), 6);

    longVecInit(&rerouting,-1);

    if (DBaddLSP(centralDB, lsp, &rerouting) < 0)
    {
	addError(CRITICAL,"Impossible to add LSP in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }	

    longVecEnd(&rerouting);

    //backup

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t1, &tz);
#endif

    lspRequestListInit(&req, -1);

    computeBackup(centralDB, &req, 0, LOCAL);

    lspRequestListEnd(&req);

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t2, &tz);
    fprintf(stderr, "Time to compute backups : %f ms\n", (t2.tv_sec - t1.tv_sec) * 1000 + 
	    (t2.tv_usec - t1.tv_usec) / 1000.0);
#endif

    // Print the DATABASE info
    // DBprintDB(centralDB);
    
    // ------------------- END TEST 1 -----------------------
    
 clean:
    
    // destroy LSP
    DBlspDestroy(lsp);

    // Destroy the DB if required
    if (centralDB != NULL)
	DBdestroy(centralDB);
    
    // Print list of error
    printErrorStack();
    
    // Destroy error manager
    errorDestroy();
    
    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);
    
    return 0;
}
