#include "database/database_api.h"
#include "computation/computation_api.h"
#include "computation/backup/backup_api.h"
#include "common/common.h"
#include "error/error.h"

#include <stdio.h>
#include <string.h>

#define LINUX
#define TIME1
#define TIMING

#if defined LINUX && defined TIMING
#include <sys/time.h>
#endif

//#include "essai.c"

int
main() 
{
    DataBase* centralDB=NULL;
    LSPRequestList reqList;
    LongVec rerouting;
    DBLabelSwitchedPath lsp;
    long i;
    long id=0;

#if defined LINUX && defined TIMING && defined TIME1
    struct timezone tz;
    struct timeval  t1,t2;
#endif

    // Init the error manager ....
    errorInit();

    // Allocate a new DB
    if ((centralDB = DBnew(0)) == NULL)
    {
	addError(CRITICAL,"Impossible to init a DataBase in %s at line %d",
		 __FILE__,__LINE__);
	goto clean;
    }

    // Configuration
    damoteConfig.predicateConfig.colorClause=FALSE;
    damoteConfig.predicateConfig.allowReroute=FALSE;
    damoteConfig.predicateConfig.capacityClause=TRUE;
   
    makeTopology(centralDB);

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);

    // Create the list of request
    printf("\n-----------------------------------------------------------------------\n");
    printf("Preparing request list\n");
    lspRequestListInit(&reqList, -1);
    makeReqList(&reqList);

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory allocated on the heap : %ld bytes\n", allocatedMemory);

    // init the lsp
    DBlspInit(&lsp);

    // Start establishement

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t1, &tz);
#endif

    printf("Nb REQ = %ld\n", reqList.size);

    for (i=0; i<reqList.size; i++) {

	long j;

	if (i%100 == 0)
	    fprintf(stderr,"%ld already done\n", i);

	// compute
	if (computePrimaryPath(centralDB, &(reqList.cont[i])) >= 0) {

	    // print
	    for (j=0; j<reqList.cont[i].path.top; j++) {
		printf("%ld ", reqList.cont[i].path.cont[j]);
	    }
	    printf("\n");

	    // make the lsp
	    longVecCopy(&(lsp.path), &(reqList.cont[i].path));
	    lsp.id = id++;
	    lsp.bw[0] = reqList.cont[i].bw[0];
	    lsp.precedence = 0;
	    lsp.type = PRIM;
	    lsp.primID = -1;
	    lsp.noContentionId = -1;

	    longVecInit(&rerouting,-1);
	    if (DBaddLSP(centralDB, &lsp, &rerouting) < 0)
	    {
		printf("Failure of LSP number %ld\n", i);
		id--;
	    }
	    longVecEnd(&rerouting);

	}
	else {
	    printf("Failure of LSP number %ld\n", i);
	}

    }

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t2, &tz);
    fprintf(stderr, "Time to establish all : %f ms\n", (t2.tv_sec - t1.tv_sec) * 1000 + 
	    (t2.tv_usec - t1.tv_usec) / 1000.0);
#endif

    // Print the DATABASE info
    DBprintDB(centralDB);
    
    // ------------------- END TEST 1 -----------------------
    
 clean:

    // Destroy the request list
    lspRequestListEnd(&reqList);
    
    // Destroy the lsp
    DBlspEnd(&lsp);

    // Destroy the DB if required
    if (centralDB != NULL)
	DBdestroy(centralDB);
    
    // Print list of error
    printErrorStack();
    
    // Destroy error manager
    errorDestroy();
    
    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);
    
    return 0;
}
