#include "computation/backup/dijkstra.h"
#include "common/common.h"
#include "error/error.h"
#include <stdio.h>
#include <stdlib.h>

#define LINUX
#define TIME1

#if defined LINUX && defined TIMING
#include <sys/time.h>
#endif

int
main() 
{
    CPPrioQueue pq;
    CPDijkNode* dn[1000];
    long i;

#if defined LINUX && defined TIMING && defined TIME1
    struct timezone tz;
    struct timeval  t1,t2;
#endif

    // Init the error manager ....
    errorInit();

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    if (CPinitPQ(&pq) < 0)
    {
	addError(CRITICAL,"Cannot init PrioQueue in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    for (i=0; i<1000; i++)
    {
	dn[i] = calloc(1, sizeof(CPDijkNode));
	dn[i]->val = i;
    }

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t1, &tz);
#endif

    for (i=0; i<1000; i++)
    {
	CPinsertPQ(&pq, dn[i], dn[i]->val);
    }

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t2, &tz);
    fprintf(stderr, "Time to insert : %f ms\n", (t2.tv_sec - t1.tv_sec) * 1000 + 
	    (t2.tv_usec - t1.tv_usec) / 1000.0);
#endif

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t1, &tz);
#endif
    
    while (CPpopTop(&pq) != NULL)
    {
	;
	// printf("Value of top == %f\n", dn->val);
    }

#if defined LINUX && defined TIMING && defined TIME1
    gettimeofday(&t2, &tz);
    fprintf(stderr, "Time to pop : %f ms\n", (t2.tv_sec - t1.tv_sec) * 1000 + 
	    (t2.tv_usec - t1.tv_usec) / 1000.0);
#endif

    for (i=0; i<1000; i++)
    {
	free(dn[i]);
    }
    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    if (CPendPQ(&pq) < 0)
    {
	addError(CRITICAL,"Cannot end PrioQueue in %s at line %d",
		 __FILE__,__LINE__);
	return -1;
    }

    // Print list of error
    printErrorStack();
    
    // Destroy error manager
    errorDestroy();

    // Check the allocated mem
    printf("\n-----------------------------------------------------------------------\n");
    printf("Memory still allocated on the heap : %ld bytes\n", allocatedMemory);

    return 0;
}
